@@ -1,6 +1,6 @@
 ;;; magit-tag.el --- Tag functionality  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2008-2024 The Magit Project Contributors
+;; Copyright (C) 2008-2025 The Magit Project Contributors
 
 ;; Author: Jonas Bernoulli <emacs.magit@jonas.bernoulli.dev>
 ;; Maintainer: Jonas Bernoulli <emacs.magit@jonas.bernoulli.dev>
@@ -62,17 +62,17 @@
   :history-key 'magit:--gpg-sign)
 
 ;;;###autoload
-(defun magit-tag-create (name rev &optional args)
-  "Create a new tag with the given NAME at REV.
+(defun magit-tag-create (name commit &optional args)
+  "Create a new tag with the given NAME at COMMIT.
 With a prefix argument annotate the tag.
 \n(git tag [--annotate] NAME REV)"
-  (interactive (list (magit-read-tag "Tag name")
+  (interactive (list (magit-completing-read "Create tag" (magit-list-tags))
                      (magit-read-branch-or-commit "Place tag on")
                      (let ((args (magit-tag-arguments)))
                        (when current-prefix-arg
                          (cl-pushnew "--annotate" args :test #'equal))
                        args)))
-  (magit-run-git-with-editor "tag" args name rev))
+  (magit-run-git-with-editor "tag" args name commit))
 
 ;;;###autoload
 (defun magit-tag-delete (tags)
@@ -84,7 +84,7 @@ defaulting to the tag at point.
   (interactive (list (if-let ((tags (magit-region-values 'tag)))
                          (magit-confirm t nil "Delete %d tags" nil tags)
                        (let ((helm-comp-read-use-marked t))
-                         (magit-read-tag "Delete tag" t)))))
+                         (magit-read-tag "Delete tag")))))
   (magit-run-git "tag" "-d" tags))
 
 ;;;###autoload
@@ -114,7 +114,7 @@ defaulting to the tag at point.
   (when tags
     (magit-call-git "tag" "-d" tags))
   (when remote-tags
-    (magit-run-git-async "push" remote (--map (concat ":" it) remote-tags))))
+    (magit-run-git-async "push" remote (mapcar (##concat ":" %) remote-tags))))
 
 (defvar magit-tag-version-regexp-alist
   '(("^[-._+ ]?snapshot\\.?$" . -4)
@@ -128,7 +128,7 @@ defaulting to the tag at point.
 See also `magit-release-tag-regexp'.")
 
 (defvar magit-release-tag-regexp "\\`\
-\\(?1:\\(?:v\\(?:ersion\\)?\\|r\\(?:elease\\)?\\)[-_]?\\)?\
+\\(?1:\\(?:v\\(?:ersion\\)?\\|r\\(?:elease\\)?\\)[-_/]?\\)?\
 \\(?2:[0-9]+\\(?:\\.[0-9]+\\)*\
 \\(?:-[a-zA-Z0-9-]+\\(?:\\.[a-zA-Z0-9-]+\\)*\\)?\\)\\'"
   "Regexp used by `magit-tag-release' to parse release tags.
@@ -167,7 +167,7 @@ that is not the case, propose a message using a reasonable format."
           (`(,pver ,ptag ,pmsg) (car (magit--list-releases)))
           (msg (magit-rev-format "%s"))
           (ver (and (string-match magit-release-commit-regexp msg)
-                    (match-string 1 msg)))
+                    (match-str 1 msg)))
           (_   (and (not ver)
                     (require (quote sisyphus) nil t)
                     (string-match magit-release-commit-regexp
@@ -185,14 +185,13 @@ that is not the case, propose a message using a reasonable format."
                                 ver)))
                 (ver
                  (concat (and (string-match magit-release-tag-regexp ptag)
-                              (match-string 1 ptag))
+                              (match-str 1 ptag))
                          ver))
-                (t
-                 (read-string
-                  (format "Create release tag (previous was %s): " ptag)
-                  ptag))))
+                ((read-string (format "Create release tag (previous was %s): "
+                                      ptag)
+                              ptag))))
           (ver (and (string-match magit-release-tag-regexp tag)
-                    (match-string 2 tag))))
+                    (match-str 2 tag))))
        (list tag
              (and (seq-some (apply-partially
                              #'string-match-p
@@ -226,13 +225,13 @@ a tag qualifies as a release tag."
     (mapcar
      #'cdr
      (nreverse
-      (cl-sort (cl-mapcan
+      (cl-sort (mapcan
                 (lambda (line)
                   (and (string-match " +" line)
                        (let ((tag (substring line 0 (match-beginning 0)))
                              (msg (substring line (match-end 0))))
                          (and (string-match magit-release-tag-regexp tag)
-                              (let ((ver (match-string 2 tag))
+                              (let ((ver (match-str 2 tag))
                                     (version-regexp-alist
                                      magit-tag-version-regexp-alist))
                                 (list (list (version-to-list ver)
@@ -245,4 +244,15 @@ a tag qualifies as a release tag."
 
 ;;; _
 (provide 'magit-tag)
+;; Local Variables:
+;; read-symbol-shorthands: (
+;;   ("and$"         . "cond-let--and$")
+;;   ("and>"         . "cond-let--and>")
+;;   ("and-let"      . "cond-let--and-let")
+;;   ("if-let"       . "cond-let--if-let")
+;;   ("when-let"     . "cond-let--when-let")
+;;   ("while-let"    . "cond-let--while-let")
+;;   ("match-string" . "match-string")
+;;   ("match-str"    . "match-string-no-properties"))
+;; End:
 ;;; magit-tag.el ends here
